package game;

import java.util.HashSet;

import network.Network;
import network.PropagationModel;

public class StackelbergGame {  
  public int ITERATIONS = 100000;
  public final Network network;
  public final PropagationModel model;
  public final HashSet<Network.Node> targetSet;
  public final HashSet<Network.Node> monitoringBaseSet;
  public final HashSet<Network.Node> initialBaseSet;
  public boolean initialSetDistribution = false;

  public StackelbergGame(Network network, PropagationModel model, HashSet<Network.Node> targetSet, HashSet<Network.Node> monitoringBaseSet, HashSet<Network.Node> initialBaseSet) {
    this.network = network;
    this.model = model;
    this.targetSet = targetSet;
    this.monitoringBaseSet = monitoringBaseSet;
    this.initialBaseSet = initialBaseSet;
  }
  
  public double attackerPayoff(HashSet<Network.Node> monitoringSet, HashSet<Network.Node> initialSet) {
    if (initialSetDistribution) {
      double numInitialNodes = initialSet.size();
      double sumPayoff = 0.0;
      for (Network.Node initialNode : initialSet) {
        HashSet<Network.Node> singleSet = new HashSet<Network.Node>();
        singleSet.add(initialNode);
        sumPayoff += model.simulatePropagation(network, singleSet, monitoringSet, targetSet, (int)(ITERATIONS / numInitialNodes));
      }
      return sumPayoff / numInitialNodes;
    }
    else
      return model.simulatePropagation(network, initialSet, monitoringSet, targetSet, ITERATIONS);
  }
  
  public double defenderPayoff(HashSet<Network.Node> monitoringSet, HashSet<Network.Node> initialSet) {
    return 1.0 - attackerPayoff(monitoringSet, initialSet);
  }  
}

