package game.attack;

import java.util.HashSet;

import network.Network;
import game.StackelbergGame;

public class GreedyAttackStrategy extends AttackStrategyAlgorithm {
  private static final GreedyAttackStrategy instance = new GreedyAttackStrategy();
  private GreedyAttackStrategy() { }
  public static GreedyAttackStrategy getInstance() { return instance; }

  @Override
  public HashSet<Network.Node> findBestResponse(StackelbergGame game, HashSet<Network.Node> monitoringSet, int budget) {
    HashSet<Network.Node> bestResponse = new HashSet<Network.Node>();
    HashSet<Network.Node> remainingNodes = new HashSet<Network.Node>(game.initialBaseSet);
    double bestResponsePayoff = 0.0;
    
    while ((!remainingNodes.isEmpty()) && (bestResponse.size() < budget)) {
      Network.Node bestNode = null;
      double maximalPayoff = -1.0;
      
      for (Network.Node node : remainingNodes) {
        bestResponse.add(node);
        double payoff = game.attackerPayoff(monitoringSet, bestResponse);
        if (payoff > maximalPayoff) {
          maximalPayoff = payoff;
          bestNode = node;
        }
        bestResponse.remove(node);
      }
      
      if (maximalPayoff > bestResponsePayoff) {
        bestResponse.add(bestNode);
        remainingNodes.remove(bestNode);
        bestResponsePayoff = maximalPayoff;
      }
      else return bestResponse;
    }
    
    return bestResponse;
  }
}
