package game.attack;

import java.util.HashSet;

import misc.Subsets;
import network.Network;
import game.StackelbergGame;

public class OptimalAttackStrategy extends AttackStrategyAlgorithm {
  private static final OptimalAttackStrategy instance = new OptimalAttackStrategy();
  private OptimalAttackStrategy() { }
  public static OptimalAttackStrategy getInstance() { return instance; }

  @Override
  public HashSet<Network.Node> findBestResponse(StackelbergGame game, HashSet<Network.Node> monitoringSet, int budget) {
    HashSet<Network.Node> bestResponse = null;
    double maximalPayoff = -1.0;
    
    for (int size = 1; size <= budget; size++)  
      for (HashSet<Network.Node> initialSet : Subsets.enumerateSubsets(game.initialBaseSet, size)) {
        double payoff = game.attackerPayoff(monitoringSet, initialSet);
        if (payoff > maximalPayoff) {
          maximalPayoff = payoff;
          bestResponse = initialSet;
        }
      }
      
    return bestResponse;
  }
}
