package game.defense;

import java.util.HashSet;
import java.util.ArrayList;
import java.util.Collections;

import network.Network;
import game.StackelbergGame;

public class Greedy1 implements DefenseResponseAlgorithm {
  private static final Greedy1 instance = new Greedy1();
  private Greedy1() { }
  public static Greedy1 getInstance() { return instance; }
  
  public String toString() { return "Greedy1"; }
  
  public HashSet<Network.Node> findDefenseResponse(StackelbergGame game, int budget, HashSet<Network.Node> initialSet) {
    HashSet<Network.Node> union = new HashSet<Network.Node>();
    ArrayList<Network.Node> initialList = new ArrayList<Network.Node>(initialSet);
    Collections.sort(initialList);
    
    for (Network.Node initialNode : initialSet) {
      HashSet<Network.Node> monitoringSet = new HashSet<Network.Node>();
      ArrayList<Network.Node> remainingNodes = new ArrayList<Network.Node>(game.monitoringBaseSet);
      Collections.sort(remainingNodes);
      HashSet<Network.Node> singleSet = new HashSet<Network.Node>();
      singleSet.add(initialNode);
      
      for (int i = 0; i < budget; i++) {
        Network.Node bestNode = null;
        double maximalPayoff = -1.0;
        
        for (Network.Node node : remainingNodes) {
          monitoringSet.add(node);
          double payoff = game.defenderPayoff(monitoringSet, singleSet);
          if (payoff > maximalPayoff) {
            maximalPayoff = payoff;
            bestNode = node;
          }
          monitoringSet.remove(node);
        }
        
        monitoringSet.add(bestNode);
        remainingNodes.remove(bestNode);
      }
      
      union.addAll(monitoringSet);
    }
    
    return union;
  }
}
