package game.defense;

import java.util.HashSet;
import java.util.HashMap;
import java.util.ArrayList;
import java.util.Collections;

import network.Network;
import game.StackelbergGame;
import game.attack.AttackStrategyAlgorithm;

public class Greedy1Strategy implements DefenseStrategyAlgorithm {
  private static final Greedy1Strategy instance = new Greedy1Strategy();
  private Greedy1Strategy() { }
  public static Greedy1Strategy getInstance() { return instance; }
  
  public String toString() { return "Greedy1"; }
 
  public HashSet<Network.Node> findDefenseStrategy(StackelbergGame game, int budget, AttackStrategyAlgorithm attackStrategy, int attackBudget) {
    ArrayList<Network.Node> initialList = new ArrayList<Network.Node>(game.initialBaseSet);
    Collections.sort(initialList);
    
    HashSet<Network.Node> union = new HashSet<Network.Node>();
    
    HashMap<Network.Node,HashSet<Network.Node>> selections = new HashMap<Network.Node,HashSet<Network.Node>>();
    HashMap<Network.Node,ArrayList<Network.Node>> remainingLists = new HashMap<Network.Node,ArrayList<Network.Node>>();
    for (Network.Node initialNode : initialList) {
      selections.put(initialNode, new HashSet<Network.Node>());
      ArrayList<Network.Node> monitoringBaseList = new ArrayList<Network.Node>(game.monitoringBaseSet);
      Collections.sort(monitoringBaseList);
      remainingLists.put(initialNode, monitoringBaseList);
    }
     
    int nextInitialNode = 0;
      
    while (true) {
      Network.Node initialNode = initialList.get(nextInitialNode++ % initialList.size());
      HashSet<Network.Node> singleSet = new HashSet<Network.Node>();
      singleSet.add(initialNode);
      
      Network.Node bestNode = null;
      double maximalPayoff = -1.0;
      HashSet<Network.Node> selection = selections.get(initialNode);

      for (Network.Node monitorNode : remainingLists.get(initialNode)) {
        selection.add(monitorNode);
        double payoff = game.defenderPayoff(selection, singleSet);
        if (payoff > maximalPayoff) {
          maximalPayoff = payoff;
          bestNode = monitorNode;
        }
        selection.remove(monitorNode);
      }
      
      selection.add(bestNode);
      remainingLists.get(initialNode).remove(bestNode);
      
      union.add(bestNode);
      if (union.size() == budget)
        return union;
    }
  }
}
