package game.defense;

import java.util.HashSet;
import java.util.ArrayList;
import java.util.Collections;

import network.Network;
import game.StackelbergGame;
import game.attack.AttackStrategyAlgorithm;

public class Greedy2Strategy implements DefenseStrategyAlgorithm {
  private static final Greedy2Strategy instance = new Greedy2Strategy();
  private Greedy2Strategy() { }
  public static Greedy2Strategy getInstance() { return instance; }
  
  public String toString() { return "Greedy2"; }
  
  public HashSet<Network.Node> findDefenseStrategy(StackelbergGame game, int budget, AttackStrategyAlgorithm attackAlgorithm, int attackBudget) {
    HashSet<Network.Node> monitoringSet = new HashSet<Network.Node>();
    ArrayList<Network.Node> remainingNodes = new ArrayList<Network.Node>(game.monitoringBaseSet);
    Collections.sort(remainingNodes);
    ArrayList<Network.Node> initialList = new ArrayList<Network.Node>(game.initialBaseSet);
    Collections.sort(initialList);
    
    int nextInitialNode = 0;
    
    for (int i = 0; i < budget; i++) {
      Network.Node bestNode = null;
      double maximalPayoff = -1.0;
      HashSet<Network.Node> singleSet = new HashSet<Network.Node>();
      singleSet.add(initialList.get((nextInitialNode++) % initialList.size()));
      
      for (Network.Node node : remainingNodes) {
        monitoringSet.add(node);
        double payoff = game.defenderPayoff(monitoringSet, singleSet);
        if (payoff > maximalPayoff) {
          maximalPayoff = payoff;
          bestNode = node;
        }
        monitoringSet.remove(node);
      }
            
      monitoringSet.add(bestNode);
      remainingNodes.remove(bestNode);
    }
    
    return monitoringSet;
  }
}
