package game.defense;

import java.util.HashSet;
import java.util.ArrayList;
import java.util.Collections;

import network.Network;
import game.StackelbergGame;


public class Greedy3 implements DefenseResponseAlgorithm {
  private static final Greedy3 instance = new Greedy3();
  private Greedy3() { }
  public static Greedy3 getInstance() { return instance; }
  
  public String toString() { return "Greedy3"; }
  
  public HashSet<Network.Node> findDefenseResponse(StackelbergGame game, int budget, HashSet<Network.Node> initialSet) {
    HashSet<Network.Node> monitoringSet = new HashSet<Network.Node>();
    ArrayList<Network.Node> remainingNodes = new ArrayList<Network.Node>(game.monitoringBaseSet);
    Collections.sort(remainingNodes);
    
    for (int i = 0; i < budget; i++) {
      Network.Node bestNode = null;
      double maximalPayoff = -1.0;
      
      for (Network.Node node : remainingNodes) {
        monitoringSet.add(node);
        double payoff = game.defenderPayoff(monitoringSet, initialSet);
        if (payoff > maximalPayoff) {
          maximalPayoff = payoff;
          bestNode = node;
        }
        monitoringSet.remove(node);
      }
      
      monitoringSet.add(bestNode);
      remainingNodes.remove(bestNode);
    }
    
    return monitoringSet;
  }
}
