package game.defense;

import java.util.HashSet;
import java.util.ArrayList;
import java.util.Collections;

import network.Network;
import game.StackelbergGame;
import game.attack.AttackStrategyAlgorithm;

public class Greedy3Strategy implements DefenseStrategyAlgorithm {
  private static final Greedy3Strategy instance = new Greedy3Strategy();
  private Greedy3Strategy() { }
  public static Greedy3Strategy getInstance() { return instance; }
  
  public String toString() { return "Greedy3"; }

  public HashSet<Network.Node> findDefenseStrategy(StackelbergGame game, int budget, AttackStrategyAlgorithm attackAlgorithm, int attackBudget) {
    HashSet<Network.Node> monitoringSet = new HashSet<Network.Node>();
    ArrayList<Network.Node> remainingNodes = new ArrayList<Network.Node>(game.monitoringBaseSet);
    Collections.sort(remainingNodes);
    
    for (int i = 0; i < budget; i++) {
      Network.Node bestNode = null;
      double maximalPayoff = -1.0;
      
      for (Network.Node node : remainingNodes) {
        monitoringSet.add(node);
        HashSet<Network.Node> initialSet = attackAlgorithm.findBestResponse(game, monitoringSet, attackBudget);
        double payoff = game.defenderPayoff(monitoringSet, initialSet);
        if (payoff > maximalPayoff) {
          maximalPayoff = payoff;
          bestNode = node;
        }
        monitoringSet.remove(node);
      }
      
      monitoringSet.add(bestNode);
      remainingNodes.remove(bestNode);
    }
    
    return monitoringSet;
  }
}
