package game.defense;

import java.util.HashSet;
import java.util.ArrayList;
import java.util.Collections;

import network.Network;
import game.StackelbergGame;


public class Greedy4 implements DefenseResponseAlgorithm {
  private static final Greedy4 instance = new Greedy4();
  private Greedy4() { }
  public static Greedy4 getInstance() { return instance; }
  
  public String toString() { return "Greedy4"; }
  
  public HashSet<Network.Node> findDefenseResponse(StackelbergGame game, int budget, HashSet<Network.Node> initialSet) {
    HashSet<Network.Node> monitoringSet = new HashSet<Network.Node>();
    ArrayList<Network.Node> remainingNodes = new ArrayList<Network.Node>(game.monitoringBaseSet);
    Collections.sort(remainingNodes);
    ArrayList<Network.Node> initialList = new ArrayList<Network.Node>(game.initialBaseSet);
    Collections.sort(initialList);
    
    for (int i = 0; i < budget; i++) {
      // Step 1.: Find a seed node with minimum coverage.
      Network.Node leastCovered = null;
      double minimalPayoff = 2.0;
      
      for (Network.Node initialNode : initialList) {
        HashSet<Network.Node> singleSet = new HashSet<Network.Node>();
        singleSet.add(initialNode);
        double payoff = game.defenderPayoff(monitoringSet, singleSet);
        if (payoff < minimalPayoff) {
          minimalPayoff = payoff;
          leastCovered = initialNode;
        }
      }
    
      // Step 2.: Find a monitoring node with maximum covering.
      Network.Node bestNode = null;
      double maximalPayoff = -1.0;
      HashSet<Network.Node> worstSingleSet = new HashSet<Network.Node>();
      worstSingleSet.add(leastCovered);
      
      for (Network.Node node : remainingNodes) {
        monitoringSet.add(node);
        double payoff = game.defenderPayoff(monitoringSet, worstSingleSet);
        if (payoff > maximalPayoff) {
          maximalPayoff = payoff;
          bestNode = node;
        }
        monitoringSet.remove(node);
      }
      
      monitoringSet.add(bestNode);
      remainingNodes.remove(bestNode);
    }
    
    return monitoringSet;
  }
}
