package game.defense;

import java.util.HashSet;

import misc.Subsets;
import network.Network;
import game.StackelbergGame;
import game.attack.AttackStrategyAlgorithm;

public class OptimalDefenseResponse implements DefenseResponseAlgorithm {
  private static final OptimalDefenseResponse instance = new OptimalDefenseResponse();
  private OptimalDefenseResponse() { }
  public static OptimalDefenseResponse getInstance() { return instance; }
  
  public String toString() { return "Optimal"; }

  public HashSet<Network.Node> findDefenseResponse(StackelbergGame game, int budget, HashSet<Network.Node> initialSet) {
    HashSet<Network.Node> optimalResponse = null;
    double maximalPayoff = -1.0;
      
    for (HashSet<Network.Node> monitoringSet : Subsets.enumerateSubsets(game.monitoringBaseSet, budget)) {      
      double payoff = game.defenderPayoff(monitoringSet, initialSet);
      if (payoff > maximalPayoff) {
        maximalPayoff = payoff;
        optimalResponse = monitoringSet;
      }
    }
      
    return optimalResponse;
  }
}
