package game.defense;

import java.util.HashSet;

import misc.Subsets;
import network.Network;
import game.StackelbergGame;
import game.attack.AttackStrategyAlgorithm;

public class OptimalDefenseStrategy implements DefenseStrategyAlgorithm {
  private static final OptimalDefenseStrategy instance = new OptimalDefenseStrategy();
  private OptimalDefenseStrategy() { }
  public static OptimalDefenseStrategy getInstance() { return instance; }
  
  public String toString() { return "Optimal"; }  
  
  public HashSet<Network.Node> findDefenseStrategy(StackelbergGame game, int budget, AttackStrategyAlgorithm attackStrategy, int attackBudget) {
    HashSet<Network.Node> optimalStrategy = null;
    double maximalPayoff = -1.0;
      
    for (HashSet<Network.Node> monitoringSet : Subsets.enumerateSubsets(game.monitoringBaseSet, budget)) {
      HashSet<Network.Node> initialSet = attackStrategy.findBestResponse(game, monitoringSet, attackBudget);
      double payoff = game.defenderPayoff(monitoringSet, initialSet);
      if (payoff > maximalPayoff) {
        maximalPayoff = payoff;
        optimalStrategy = monitoringSet;
      }
    }
      
    return optimalStrategy;
  }
}
