package misc;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.Collection;
import java.util.Random;

public abstract class Subsets {
  static private <T> ArrayList<HashSet<T>> _enumerateSubsets(ArrayList<T> baseSet, int size) {
    if (size == 1) {
      ArrayList<HashSet<T>> subsets = new ArrayList<HashSet<T>>();
      for (T element : baseSet) {
        HashSet<T> subset = new HashSet<T>();
        subset.add(element); 
        subsets.add(subset);
      }
      return subsets;
    }
    else if (baseSet.size() == size) {
      ArrayList<HashSet<T>> subsets = new ArrayList<HashSet<T>>();
      subsets.add(new HashSet<T>(baseSet));
      return subsets;
    } 
    else {
      T element = baseSet.remove(0);
      ArrayList<HashSet<T>> subsets = _enumerateSubsets(baseSet, size);
      for (HashSet<T> subset : _enumerateSubsets(baseSet, size - 1)) {
        subset.add(element);
        subsets.add(subset);
      }
      baseSet.add(0, element);
      return subsets;
    }
  }
  
  static public <T> ArrayList<HashSet<T>> enumerateSubsets(Collection<T> baseSet, int size) {
    return _enumerateSubsets(new ArrayList<T>(baseSet), size);
  } 
  
  static public <T> ArrayList<HashSet<T>> enumerateSubsets(Collection<T> baseSet) {
    ArrayList<T> baseSetCopy = new ArrayList<T>(baseSet);
    ArrayList<HashSet<T>> subsets = new ArrayList<HashSet<T>>();
    subsets.add(new HashSet<T>()); // empty set
    
    for (int size = 1; size <= baseSet.size(); size++)
      subsets.addAll(_enumerateSubsets(baseSetCopy, size));
      
   return subsets;
  } 
  
  static public <T> HashSet<T> randomSubset(Collection<T> baseSet, int size, Random random) {
    ArrayList<T> baseList = new ArrayList<T>(baseSet);
    HashSet<T> subset = new HashSet<T>();
    
    while (subset.size() < size)
      subset.add(baseList.get(Math.abs(random.nextInt()) % baseList.size()));
      
    return subset;
  }
}
