package network;

import java.util.Collection;
import java.util.HashSet;
import java.util.ArrayList;
import java.util.HashMap;

public class Network {
  private int nextNodeID = 0;
  
  public class Node implements Comparable<Node> {
    public final int ID = nextNodeID++;
    
    public Collection<Node> getNeighbors() {
      return Network.this.neighbors.get(this);
    }
    
    public String toString() {
      return Integer.toString(ID);
    }
    
    @Override
    public int compareTo(Node other) {
      return Integer.compare(ID, other.ID);
    }
  }
  
  public static final Network.Node[] emptyNodeArray = new Network.Node[0]; // for calling toArray

  private ArrayList<Node> nodes = new ArrayList<Node>();
  private HashMap<Node,ArrayList<Node>> neighbors = new HashMap<Node,ArrayList<Node>>();
  private HashMap<Node,HashMap<Node,Double>> weights = new HashMap<Node,HashMap<Node,Double>>();
  
  // construction
  
  public Node addNode() {
    Node node = new Node();
    nodes.add(node);
    neighbors.put(node, new ArrayList<Node>());
    weights.put(node, new HashMap<Node,Double>());
    return node;
  }
  
  public void addArc(Node from, Node to, double weight) {
    neighbors.get(from).add(to);
    weights.get(from).put(to, weight);
  }
  
  public void addEdge(Node a, Node b, double weight) {
    addArc(a, b, weight);
    addArc(b, a, weight);
  }
  
  // getters
  
  public double getWeight(Node from, Node to) {
    return weights.get(from).get(to);
  }
  
  public Collection<Node> getNodes() {
    return nodes;
  }
  
  public Node getNode(int nodeID) {
    return nodes.get(nodeID);
  }
  
  // performance, yay!
  
  public int[][] _arcs = null;
  public int[] _arcFrom = null;
  public int[] _arcTo = null;
  public double[] _arcWeight = null;
  
  public void createArrays() {
    Node[] _nodes = nodes.toArray(emptyNodeArray); 
    int numArcs = 0;
    
    for (int nodeID = 0; nodeID < _nodes.length; nodeID++) {
      Node node = _nodes[nodeID];
      assert node.ID == nodeID;      
      for (Node neighbor : neighbors.get(node))
        numArcs++;
    }
    
    _arcs = new int[_nodes.length][];
    _arcFrom = new int[numArcs];
    _arcTo = new int[numArcs];
    _arcWeight = new double[numArcs];
    int arcID = 0;

    for (Node node : nodes) {
      _arcs[node.ID] = new int[node.getNeighbors().size()];
      
      int i = 0;
      for (Node neighbor : node.getNeighbors()) { 
        _arcs[node.ID][i++] = arcID;
        _arcFrom[arcID] = node.ID;
        _arcTo[arcID] = neighbor.ID;
        _arcWeight[arcID++] = getWeight(node, neighbor);
      }
    }
  }
}

