package network;

import java.util.Set;
import java.util.Random;

public abstract class PropagationModel {
  protected static final int REGULAR = 0;
  protected static final int INITIAL = 1;
  protected static final int MONITORING = 2;
  protected static final int TARGET = 3;
    
  protected abstract boolean simulatePropagation(Network network, Set<Network.Node> initialSet, Set<Network.Node> monitoringSet, Set<Network.Node> targetSet, Random random);
  
  public double simulatePropagation(Network network, Set<Network.Node> initialSet, Set<Network.Node> monitoringSet, Set<Network.Node> targetSet, int iterations) {
    int successes = 0;
    
    for (int i = 0; i < iterations; i++)
      if (simulatePropagation(network, initialSet, monitoringSet, targetSet, new Random(i)))
        successes++;
    
    return successes / (double)iterations;
  }
}

