package network;

import java.util.Set;
import java.util.HashSet;
import java.util.ArrayList;
import java.util.Random;
import java.util.Arrays;
import java.util.Collections;

public class RepeatedIndependentCascades extends PropagationModel {
  private static final RepeatedIndependentCascades instance = new RepeatedIndependentCascades();
  private RepeatedIndependentCascades() {}
  public static RepeatedIndependentCascades getInstance() { return instance; } 
      
  @Override
  protected boolean simulatePropagation(Network network, Set<Network.Node> initialSet, Set<Network.Node> monitoringSet, Set<Network.Node> targetSet, Random random) {
    int numNodes = network.getNodes().size();
    int[] activeEdges = new int[network._arcFrom.length];
    int numActiveEdges = 0;
    int[] pendingEdges = new int[network._arcFrom.length];
    int numPendingEdges = 0;
    boolean[] infected = new boolean[numNodes];
    int[] nodeType = new int[numNodes]; // 0: regular, 1: initial, 2: monitoring, 3: target

    ArrayList<Integer> initialEdges = new ArrayList<Integer>();

    for (Network.Node node : initialSet) {
      int nodeID = node.ID;
      infected[nodeID] = true;
      for (int arcID : network._arcs[nodeID])
        initialEdges.add(arcID);
    }

    Collections.sort(initialEdges); // since hashsets are not ordered, we have to sort the edges to make the outcome consistent

    for (Integer arcID : initialEdges)
      activeEdges[numActiveEdges++] = arcID;

    for (Network.Node node : monitoringSet)
      nodeType[node.ID] = MONITORING;      
    for (Network.Node node : targetSet)
      nodeType[node.ID] = TARGET;   
      
    while (true) {
      boolean detection = false;
      
      for (int i = 0; i < numActiveEdges; i++) {
        int arcID = activeEdges[i];
        int toNodeID = network._arcTo[arcID];
        
        if (!infected[toNodeID]) {
          if (random.nextDouble() < network._arcWeight[arcID]) {
            if (nodeType[toNodeID] == TARGET)
              return true; // epidemic reached target node first
            if (nodeType[toNodeID] == MONITORING)
              detection = true; // epidemic reaches monitoring node
              
            infected[toNodeID] = true;
            for (int neighborArcID : network._arcs[toNodeID])            
              if (!infected[network._arcTo[neighborArcID]])
                pendingEdges[numPendingEdges++] = neighborArcID;
          }
          else
            pendingEdges[numPendingEdges++] = arcID;
        }
      }

      if (detection)
        return false; // epidemic reached monitoring node
      if (numPendingEdges == 0)
        return false; // epidemic died
      
      int[] tmp = activeEdges; // avoid allocating new array
      activeEdges = pendingEdges;
      numActiveEdges = numPendingEdges;
      pendingEdges = tmp;
      numPendingEdges = 0;   
    }
  }
}

